package com.qmth.distributed.print.business.templete.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.Gson;
import com.qmth.distributed.print.business.bean.dto.excel.UserImportDto;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.ExamCardStatusEnum;
import com.qmth.distributed.print.business.enums.ExamStatusEnum;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.service.TaskLogicService;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.business.util.excel.ExcelError;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import com.qmth.distributed.print.common.util.Base64Util;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Description: 任务处理逻辑impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
@Service
public class TaskLogicServiceImpl implements TaskLogicService {

    @Resource
    SysUserService sysUserService;

    @Resource
    BasicSchoolService basicSchoolService;

    @Resource
    CommonService commonService;

    @Resource
    ExamDetailService detailService;

    @Resource
    ExamDetailCourseService detailCourseService;

    @Resource
    ExamPrintPlanService examPrintPlanService;

    @Resource
    ExamTaskService examTaskService;

    @Resource
    ExamTaskDetailService examTaskDetailService;

    @Resource
    ExamCardService examCardService;

    @Resource
    ExamCardDetailService examCardDetailService;

    @Resource
    ExamStudentService examStudentService;

    /**
     * 执行导入用户逻辑
     *
     * @param finalList
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> executeUserImportLogic(List<LinkedMultiValueMap<Integer, Object>> finalList, Map<String, Object> map) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        Gson gson = new Gson();
        LinkedMultiValueMap<Long, SysUser> tbUserLinkedMultiValueMap = new LinkedMultiValueMap<>();
        for (int i = 0; i < finalList.size(); i++) {
            LinkedMultiValueMap<Integer, Object> excelMap = finalList.get(i);
            List<Object> userImportDtoList = excelMap.get(i);
            for (int y = 0; y < userImportDtoList.size(); y++) {
                UserImportDto userImportDto = (UserImportDto) userImportDtoList.get(y);
                SysUser sysUser = gson.fromJson(gson.toJson(userImportDto), SysUser.class);

                //查用户登录名是否重复
                QueryWrapper<SysUser> sysUserQueryWrapper = new QueryWrapper<>();
                sysUserQueryWrapper.lambda().eq(SysUser::getLoginName, userImportDto.getLoginName());
                long count = sysUserService.count(sysUserQueryWrapper);
                if (count > 0) {
                    throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[登录名]重复")));
                }
                //查学校是否存在
                if (Objects.nonNull(userImportDto.getSchoolCode())) {
                    QueryWrapper<BasicSchool> basicSchoolQueryWrapper = new QueryWrapper<>();
                    basicSchoolQueryWrapper.lambda().eq(BasicSchool::getCode, userImportDto.getSchoolCode());
                    BasicSchool basicSchool = basicSchoolService.getOne(basicSchoolQueryWrapper);
                    if (Objects.isNull(basicSchool)) {
                        throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[学校代码]不存在")));
                    }
                    sysUser.setSchoolId(basicSchool.getId());
                }
                sysUser.setInsertInfo(requestUser.getId());
                sysUser.setPassword(Base64Util.encode(userImportDto.getPassword().getBytes()));
                tbUserLinkedMultiValueMap.add(userImportDto.getSysRole().getId(), sysUser);
            }
        }
        tbUserLinkedMultiValueMap.forEach((k, v) -> {
            sysUserService.saveBatch(v);
            v.forEach(s -> {
                commonService.addUserRolePrivilege(s, k);
            });
        });
        return map;
    }

    /**
     * 创建pdf逻辑
     *
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> executeCreatePdfLogic(Map<String, Object> map) {
        TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
        SysUser sysUser = (SysUser) map.get(SystemConstant.USER);
        //查询printPlan
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(tbTask.getPrintPlanId());
        if (Objects.isNull(examPrintPlan)) {
            throw ExceptionResultEnum.ERROR.exception("印刷计划为空");
        }

        //查询examDetail
        QueryWrapper<ExamDetail> examDetailQueryWrapper = new QueryWrapper<>();
        examDetailQueryWrapper.lambda().eq(ExamDetail::getSchoolId, sysUser.getSchoolId())
                .eq(ExamDetail::getPrintPlanId, tbTask.getPrintPlanId());
        ExamDetail examDetail = detailService.getOne(examDetailQueryWrapper);
        if (Objects.isNull(examDetail)) {
            throw ExceptionResultEnum.ERROR.exception("考务计划为空");
        }

        //查询examDetailCourse
        QueryWrapper<ExamDetailCourse> examDetailCourseQueryWrapper = new QueryWrapper<>();
        examDetailCourseQueryWrapper.lambda().eq(ExamDetailCourse::getExamDetailId, examDetail.getId());
        List<ExamDetailCourse> examDetailCourseList = detailCourseService.list(examDetailCourseQueryWrapper);

        for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
            //查询试卷
            QueryWrapper<ExamTask> examTaskQueryWrapper = new QueryWrapper<>();
            examTaskQueryWrapper.lambda().eq(ExamTask::getSchoolId, sysUser.getSchoolId())
                    .eq(ExamTask::getCourseCode, examDetailCourse.getCourseCode())
                    .eq(ExamTask::getCourseName, examDetailCourse.getCourseName())
                    .eq(ExamTask::getPaperNumber, examDetailCourse.getPaperNumber())
                    .eq(ExamTask::getEnable, true)
                    .eq(ExamTask::getStatus, ExamStatusEnum.FINISH);
            List<ExamTask> examTaskList = examTaskService.list(examTaskQueryWrapper);
            if (Objects.isNull(examTaskList) || examTaskList.size() == 0) {
                throw ExceptionResultEnum.ERROR.exception("命题任务为空");
            }
            Set<Long> examTaskIds = examTaskList.stream().map(s -> s.getId()).collect(Collectors.toSet());

            QueryWrapper<ExamTaskDetail> examTaskDetailQueryWrapper = new QueryWrapper<>();
            examTaskDetailQueryWrapper.lambda().in(ExamTaskDetail::getExamTaskId, examTaskIds)
                    .eq(ExamTaskDetail::getEnable, true);
            List<ExamTaskDetail> examTaskDetailList = examTaskDetailService.list(examTaskDetailQueryWrapper);
            Set<Long> examTaskDetailIds = examTaskDetailList.stream().map(s -> s.getId()).collect(Collectors.toSet());

            //查询题卡
            QueryWrapper<ExamCard> examCardQueryWrapper = new QueryWrapper<>();
            examCardQueryWrapper.lambda().eq(ExamCard::getSchoolId, sysUser.getSchoolId())
                    .eq(ExamCard::getCourseCode, examDetailCourse.getCourseCode())
                    .eq(ExamCard::getCourseName, examDetailCourse.getCourseName())
                    .eq(ExamCard::getStatus, ExamCardStatusEnum.SUBMIT);
            List<ExamCard> examCardList = examCardService.list(examCardQueryWrapper);
            if (Objects.isNull(examCardList) || examCardList.size() == 0) {
                throw ExceptionResultEnum.ERROR.exception("题卡为空");
            }
            Set<Long> examCardIds = examCardList.stream().map(s -> s.getId()).collect(Collectors.toSet());

            QueryWrapper<ExamCardDetail> examCardDetailQueryWrapper = new QueryWrapper<>();
            examCardDetailQueryWrapper.lambda().in(ExamCardDetail::getCardId, examCardIds);
            List<ExamCardDetail> examCardDetailList = examCardDetailService.list(examCardDetailQueryWrapper);

            //查询考生
            QueryWrapper<ExamStudent> examStudentQueryWrapper = new QueryWrapper<>();
            examStudentQueryWrapper.lambda().eq(ExamStudent::getSchoolId, sysUser.getSchoolId())
                    .in(ExamStudent::getExamDetailCourseId, examTaskDetailIds);
            List<ExamStudent> examStudentList = examStudentService.list(examStudentQueryWrapper);
        }
        return map;
    }
}
